from __future__ import absolute_import
from re import search
from allure_commons.model2 import Status, Label, Parameter, Link
from allure_commons.types import LabelType
from allure_robotframework.types import RobotStatus
from allure_commons.mapping import parse_tag, labels_set, allure_tag_sep


def get_allure_status(status):
    if status == RobotStatus.PASSED:
        return Status.PASSED
    elif status == RobotStatus.SKIPPED:
        return Status.SKIPPED
    else:
        return Status.FAILED


def get_allure_parameters(parameters):
    return [Parameter(name="arg{}".format(i + 1), value=param) for i, param in enumerate(parameters)]


def get_allure_suites(longname):
    """
    >>> get_allure_suites('Suite1.Test')
    [Label(name='suite', value='Suite1')]
    >>> get_allure_suites('Suite1.Suite2.Test') # doctest: +NORMALIZE_WHITESPACE
    [Label(name='suite', value='Suite1'), Label(name='subSuite', value='Suite2')]
    >>> get_allure_suites('Suite1.Suite2.Suite3.Test') # doctest: +NORMALIZE_WHITESPACE
    [Label(name='parentSuite', value='Suite1'),
    Label(name='suite', value='Suite2'),
    Label(name='subSuite', value='Suite3')]
    """
    labels = []
    suites = longname.split('.')
    if len(suites) > 3:
        labels.append(Label(LabelType.PARENT_SUITE, suites.pop(0)))
    labels.append(Label(LabelType.SUITE, suites.pop(0)))
    if len(suites) > 1:
        labels.append(Label(LabelType.SUB_SUITE, '.'.join(suites[:-1])))
    return labels


def allure_tags(attributes):
    return [Label(LabelType.TAG, tag) for tag in attributes.get('tags', ()) if not allure_tag_sep(tag)]


def allure_labels(tags):
    parsed = [parse_tag(item) for item in tags]
    return labels_set(list(filter(lambda x: isinstance(x, Label),  parsed)))


def allure_links(attributes, prefix):
    tags = attributes.get('tags', ())

    def is_link(link):
        return link.startswith("{link}:".format(link=prefix))

    def parse_link(link):
        lnk_val = link.split(':', 1)[1] or 'unknown'
        lnk_label = search(r'\[.+\]', lnk_val)
        if lnk_label:
            lnk_label = lnk_label.group(0)
            lnk_val = lnk_val.strip(lnk_label)
            lnk_label = lnk_label.strip('[]')
        else:
            lnk_label = lnk_val

        return {'name': lnk_label, 'value': lnk_val}

    return [Link(type=prefix, url=parse_link(tag).get('value'), name=parse_link(tag).get('name')) for tag in tags if
            is_link(tag)]



from functools import partial
from hamcrest import assert_that
from hamcrest import not_, anything
from allure_commons_test.report import has_test_case
from allure_commons_test.result import has_description
from allure_commons_test.result import has_step
from allure_commons_test.result import with_status
from allure_commons_test.result import has_parameter
from allure_commons_test.result import has_link
from allure_commons_test.result import has_status_details
from allure_commons_test.result import with_message_contains
from allure_commons_test.result import with_trace_contains
from allure_commons_test.result import has_attachment
from allure_commons_test.label import has_severity
from allure_commons_test.label import has_tag
from allure_commons_test.label import has_label
from allure_commons_test.container import has_container
from allure_commons_test.container import has_before, has_after


def match(matcher, *args):
    for i, arg in enumerate(args):
        if not hasattr(arg, '__call__'):
            matcher = partial(matcher, arg)
        else:
            matcher = partial(matcher, match(arg, *args[i+1:]))
            break
    return matcher()


def should_has_test_case(allure_report, test_case):
    test_case_matcher = partial(match, has_test_case, test_case)
    assert_that(allure_report, test_case_matcher())
    return allure_report, test_case_matcher


def should_has_tag(context, tag):
    allure_report, test_case_matcher = context
    tag_matcher = partial(test_case_matcher, has_tag, tag)
    assert_that(allure_report, tag_matcher())


def should_not_has_tag(conetxt, tag):
    allure_report, test_case_matcher = conetxt
    tag_matcher = partial(test_case_matcher, not_, has_tag, tag)
    assert_that(allure_report, tag_matcher())


def should_has_severity(context, severity):
    allure_report, test_case_matcher = context
    severity_matcher = partial(test_case_matcher, has_severity, severity)
    assert_that(allure_report, severity_matcher())


def should_not_has_severity(context):
    allure_report, test_case_matcher = context
    severity_matcher = partial(test_case_matcher, not_, has_severity, anything)
    assert_that(allure_report, severity_matcher())


def should_has_step(context, step):
    allure_report, item_matcher = context
    step_matcher = partial(item_matcher, has_step, step)
    assert_that(allure_report, step_matcher())
    return allure_report, step_matcher


def should_not_has_step(context, step):
    allure_report, item_matcher = context
    step_matcher = partial(item_matcher, not_, has_step, step)
    assert_that(allure_report, step_matcher())


def should_has_attachment(context, attach_type=None, name=None):
    allure_report, item_matcher = context
    matcher = partial(item_matcher, has_attachment,  attach_type, name)
    assert_that(allure_report, matcher())


def should_has_description(context, description):
    allure_report, test_case_matcher = context
    description_matcher = partial(test_case_matcher, has_description, description)
    assert_that(allure_report, description_matcher())


def should_has_before_fixture(context, fixture):
    allure_report, test_case_matcher = context
    fixture_matcher = partial(test_case_matcher, has_container, allure_report,  has_before, fixture)
    assert_that(allure_report, fixture_matcher())
    return allure_report, fixture_matcher


def should_has_after_fixture(context, fixture):
    allure_report, test_case_matcher = context
    fixture_matcher = partial(test_case_matcher, has_container, allure_report,  has_after, fixture)
    assert_that(allure_report, fixture_matcher())
    return allure_report, fixture_matcher


def should_has_status(context, status):
    allure_report, item_matcher = context
    status_matcher = partial(item_matcher, with_status, status)
    assert_that(allure_report, status_matcher())


def should_has_parameter(context, name, value):
    allure_report, item_matcher = context
    matcher = partial(item_matcher, has_parameter, name, value)
    assert_that(allure_report, matcher())


def should_has_status_detail_with_message(conext, message):
    allure_report, item_matcher = conext
    matcher = partial(item_matcher, has_status_details, with_message_contains, message)
    assert_that(allure_report, matcher())


def should_has_status_detail_with_traceback(conext, traceback):
    allure_report, item_matcher = conext
    matcher = partial(item_matcher, has_status_details, with_trace_contains, traceback)
    assert_that(allure_report, matcher())


def should_has_link(context, link, link_type=None, link_name=None):
    allure_report, test_case_matcher = context
    matcher = partial(test_case_matcher, has_link, link, link_type, link_name)
    assert_that(allure_report, matcher())


def should_has_label(context, name, value):
    allure_report, test_case_matcher = context
    matcher = partial(test_case_matcher, has_label, name, value)
    assert_that(allure_report, matcher())



import os
from tempfile import mkdtemp, mkstemp
from robot import run
from multiprocessing import Process
from allure_commons_test.report import AllureReport


def run_robot_with_allure(*args, **kwargs):
    root = os.path.abspath(os.path.join(__file__, "..", ".."))
    targets = map(lambda target: os.path.join(root, target), args)
    tmp_path = mkdtemp(dir=os.environ.get('TEST_TMP', '/tmp'))

    if "testplan" in kwargs:
        # kwargs.pop("testplan")
        kwargs["prerunmodifier"] = "allure_robotframework.testplan"
        file, filename = mkstemp(suffix=".json", dir=tmp_path)
        os.environ["ALLURE_TESTPLAN_PATH"] = filename
        with os.fdopen(file, 'w') as tmp:
            tmp.write(kwargs["testplan"])

    def run_robot(path, **kw):

        # ToDo: fix it (_core not works correctly with multiprocessing)
        # import six
        # import allure_commons
        # if six.PY2:
        #     reload(allure_commons._core)
        # else:
        #     import importlib
        #     importlib.reload(allure_commons._core)
        #
        #

        from allure_robotframework import allure_robotframework

        listener = allure_robotframework(logger_path=tmp_path)
        stdout_file = os.path.abspath(os.path.join(tmp_path, "..", "stdout.txt"))
        output_path = os.path.abspath(os.path.join(tmp_path, ".."))

        with open(stdout_file, 'w+') as stdout:
            options = {"listener": listener, "outputdir": output_path, "stdout": stdout, "extension": "rst"}
            options.update(kw)
            run(path, **options)

    robot_process = Process(target=run_robot, args=targets, kwargs=kwargs)
    robot_process.start()
    robot_process.join()

    os.environ.pop("ALLURE_TESTPLAN_PATH", None)

    return AllureReport(tmp_path)



import os
from setuptools import setup

PACKAGE = "allure-robotframework"

classifiers = [
    'Development Status :: 5 - Production/Stable',
    'Framework :: Robot Framework',
    'Framework :: Robot Framework :: Tool',
    'Intended Audience :: Developers',
    'License :: OSI Approved :: Apache Software License',
    'Topic :: Software Development :: Quality Assurance',
    'Topic :: Software Development :: Testing',
]

setup_requires = [
    "setuptools_scm"
]

install_requires = [
]


def prepare_version():
    from setuptools_scm import get_version
    configuration = {"root": "..", "relative_to": __file__}
    version = get_version(**configuration)
    install_requires.append("allure-python-commons=={version}".format(version=version))
    return configuration


def get_readme(fname):
    return open(os.path.join(os.path.dirname(__file__), fname)).read()


if __name__ == '__main__':
    setup(
        name=PACKAGE,
        use_scm_version=prepare_version,
        description="Allure Robot Framework integration",
        license="Apache-2.0",
        install_requires=install_requires,
        setup_requires=setup_requires,
        keywords="allure reporting robotframework",
        packages=['allure_robotframework', 'AllureLibrary'],
        package_dir={"allure_robotframework": "src/listener", 'AllureLibrary': 'src/library'},
        py_modules=['allure_robotframework'],
        url="https://github.com/allure-framework/allure-python",
        author="Sergey Khomutinin",
        author_email="skhomuti@gmail.com",
        long_description=get_readme('README.rst'),
        classifiers=classifiers,
    )



import os
from tempfile import mkdtemp


class ForeignLibrary(object):

    def capture_page_screenshot(self):
        tmp = mkdtemp()
        screenshot_path = os.path.join(tmp, 'screenshot.txt')
        with open(screenshot_path, 'w+') as screenshot:
            screenshot.write("Grab some beer and be happy~(^o^)-c[~]")
            return screenshot_path


foreign_library = ForeignLibrary



import allure
from robot.libraries.BuiltIn import BuiltIn


class ForeignLibraryHelper(object):
    ROBOT_LIBRARY_SCOPE = "TEST SUITE"
    ROBOT_LISTENER_API_VERSION = 2

    def __init__(self):
        self.ROBOT_LIBRARY_LISTENER = self

    def _start_suite(self, name, attrs):
        BuiltIn().set_library_search_order('foreign_library_helper')

    def capture_page_screenshot(self):
        helper_library = BuiltIn().get_library_instance('foreign_library')
        path = helper_library.capture_page_screenshot()
        allure.attach.file(path, name="screenshot", attachment_type=allure.attachment_type.TEXT)
        return path


foreign_library_helper = ForeignLibraryHelper
